@@ -301,6 +301,10 @@ pub enum SessionInput {
     /// in form of `DISTANT DATA <host> <port> <auth key>`
     Pipe,
 
+    /// Session is stored and retrieved from the initializeOptions of the initialize request
+    /// that is first sent for an LSP through
+    Lsp,
+
     /// Session isn't directly available but instead there is a process listening
     /// on a unix socket that will forward requests and responses
     #[cfg(unix)]
@@ -408,6 +412,10 @@ pub struct LaunchSubcommand {
     #[structopt(short, long)]
     pub identity_file: Option<PathBuf>,
 
+    /// If specified, will not launch distant using a login shell but instead execute it directly
+    #[structopt(long)]
+    pub no_shell: bool,
+
     /// Port to use for sshing into the remote machine
     #[structopt(short, long, default_value = "22")]
     pub port: u16,
