@@ -7,6 +7,7 @@ use crate::{
         state::ServerState,
         utils,
     },
+    ExitCode, ExitCodeError,
 };
 use derive_more::{Display, Error, From};
 use fork::{daemon, Fork};
@@ -29,6 +30,16 @@ pub enum Error {
     IoError(io::Error),
 }
 
+impl ExitCodeError for Error {
+    fn to_exit_code(&self) -> ExitCode {
+        match self {
+            Self::ConvertToIpAddrError(_) => ExitCode::NoHost,
+            Self::ForkError => ExitCode::OsErr,
+            Self::IoError(x) => x.to_exit_code(),
+        }
+    }
+}
+
 pub fn run(cmd: ListenSubcommand, opt: CommonOpt) -> Result<(), Error> {
     if cmd.daemon {
         // NOTE: We keep the stdin, stdout, stderr open so we can print out the pid with the parent
