@@ -17,6 +17,13 @@ pub const MAX_PIPE_CHUNK_SIZE: usize = 1024;
 /// Represents the length of the salt to use for encryption
 pub const SALT_LEN: usize = 16;
 
+/// Test-only constants
+#[cfg(test)]
+pub mod test {
+    pub const BUFFER_SIZE: usize = 100;
+    pub const TENANT: &str = "test-tenant";
+}
+
 lazy_static::lazy_static! {
     pub static ref TIMEOUT_STR: String = TIMEOUT.to_string();
 
